﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NMeCab.Core
{
    public static class StrUtils
    {
        /// <summary>
        /// バイト配列の中から終端が\0で表された文字列を取り出す。
        /// バイト配列の長さはInt32.MaxValueを超えていても良い。
        /// </summary>
        /// <param name="bytes">バイト配列</param>
        /// <param name="enc">文字エンコーディング</param>
        /// <returns>文字列（\0は含まない）</returns>
        public static string GetString(byte[] bytes, Encoding enc)
        {
            return StrUtils.GetString(bytes, 0L, enc);
        }

        /// <summary>
        /// バイト配列の中から終端が\0で表された文字列を取り出す。
        /// バイト配列の長さはInt32.MaxValueを超えていても良い。
        /// </summary>
        /// <param name="bytes">バイト配列</param>
        /// <param name="index">オフセット位置</param>
        /// <param name="enc">文字エンコーディング</param>
        /// <returns>文字列（\0は含まない）</returns>
        public unsafe static string GetString(byte[] bytes, long offset, Encoding enc)
        {
            fixed (byte* pBytes = bytes)
            {
                return StrUtils.GetString(pBytes + offset, enc);
            }
        }

        /// <summary>
        /// バイト配列の中から終端が\0で表された文字列を取り出す。
        /// バイト配列の長さはInt32.MaxValueを超えていても良い。
        /// </summary>
        /// <param name="bytes">デコードする最初のバイトへのポインタ</param>
        /// <param name="enc">文字エンコーディング</param>
        /// <returns>文字列（\0は含まない）</returns>
        public unsafe static string GetString(byte* bytes, Encoding enc)
        {
            //バイト長のカウント
            int byteCount = 0;
            while (*bytes != 0x0)
            {
                checked { byteCount++; } //文字列のバイト長がInt32.MaxValueを超えたならエラー
                bytes++;
            }
            bytes -= byteCount;

            //生成されうる最大文字数のバッファを確保
            int maxCharCount = enc.GetMaxCharCount(byteCount);
            fixed (char* buff = new char[maxCharCount])
            {
                //バイト配列を文字列にデコード
                int len = enc.GetChars(bytes, byteCount, buff, maxCharCount);
                return new string(buff, 0, len);
            }
        }
    }
}
